// Copyright 1998-2016 Epic Games, Inc. All Rights Reserved.

#include "AndroidKeyboardPrivatePCH.h"

#if PLATFORM_ANDROID
#include "../../../Core/Public/Android/AndroidApplication.h"
#include "../../../Launch/Public/Android/AndroidJNI.h"
#include <android/log.h>

#define LOG_TAG "KeyboardLOG"

JNIEnv* FAndroidKeyboardModule::ENV = NULL;

static jmethodID AndroidThunkJava_ShowKeyboard;
static jmethodID AndroidThunkJava_IsKeyboardOpen;
static jmethodID AndroidThunkJava_HideKeyboard;
static jmethodID AndroidThunkJava_GetKeyboardHeight;
static jmethodID AndroidThunkJava_SetText;

static jmethodID AndroidThunkJava_SetCursor;
static jmethodID AndroidThunkJava_GetCursor;

extern int AndroidThunkCpp_GetKeyboardHeight();
extern void AndroidThunkCpp_OpenKeyboard(int ref, int type);
extern bool AndroidThunkCpp_IsKeyboardOpen();
extern void AndroidThunkCpp_HideKeyboard(int ref);
extern void AndroidThunkCpp_SetText(const FString &rTxt);

static void AndroidThunkCpp_SetCursor(int iPos);
static int AndroidThunkCpp_GetCursor();

KeyboardResultFn FAndroidKeyboardModule::m_KeyboardResult;
int FAndroidKeyboardModule::m_KeyboardHeight;
#endif

#define LOCTEXT_NAMESPACE "FAndroidKeyboardModule"

void FAndroidKeyboardModule::StartupModule()
{
	// This code will execute after your module is loaded into memory; the exact timing is specified in the .uplugin file per-module
#if PLATFORM_ANDROID
	JNIEnv* env = FAndroidApplication::GetJavaEnv();
	if (!env)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: FAndroidKeyboardModule no Java ENV");
		return;
	}

	ENV = env;

	AndroidThunkJava_IsKeyboardOpen = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_IsKeyboardOpen", "()Z", false);
	if (!AndroidThunkJava_IsKeyboardOpen)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_IsKeyboardOpen Method cant be found");
		return;
	}

	AndroidThunkJava_ShowKeyboard = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_ShowKeyboard", "(II)V", false);
	if (!AndroidThunkJava_ShowKeyboard)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_ShowKeyboard Method cant be found");
		return;
	}

	AndroidThunkJava_HideKeyboard = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_HideKeyboard", "(I)V", false);
	if (!AndroidThunkJava_HideKeyboard)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_HideKeyboard Method cant be found");
		return;
	}

	AndroidThunkJava_GetKeyboardHeight = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_GetKeyboardHeight", "()I", false);
	if (!AndroidThunkJava_GetKeyboardHeight)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_GetKeyboardHeight Method cant be found");
		return;
	}

	AndroidThunkJava_SetCursor = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_SetCursor", "(I)V", false);
	if (!AndroidThunkJava_SetCursor)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_SetCursor Method cant be found");
		return;
	}
	
	AndroidThunkJava_GetCursor = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_GetCursor", "()I", false);
	if (!AndroidThunkJava_GetCursor)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_GetCursor Method cant be found");
		return;
	}

	AndroidThunkJava_SetText = FJavaWrapper::FindMethod(ENV, FJavaWrapper::GameActivityClassID, "AndroidThunkJava_SetText", "(Ljava/lang/String;)V", false);
	if (!AndroidThunkJava_SetText)
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ERROR: AndroidThunkJava_SetText Method cant be found");
		return;		
	}

	__android_log_print(ANDROID_LOG_INFO, LOG_TAG, "LOOKHERE: ***Keyboard module load success***");
#endif
}

void FAndroidKeyboardModule::ShutdownModule()
{
	// This function may be called during shutdown to clean up your module.  For modules that support dynamic reloading,
	// we call this function before unloading the module.
	
}

#if PLATFORM_ANDROID
void FAndroidKeyboardModule::OpenKeyboard(const KeyboardResultFn &rFn, const FString &rTxt, int ref, int type)
{
	__android_log_print(ANDROID_LOG_INFO, LOG_TAG, "LOOKHERE: ***FAndroidKeyboardModule::OpenKeyboard(%d)", ref);
	m_KeyboardResult = rFn;
	AndroidThunkCpp_OpenKeyboard(ref, type);
	AndroidThunkCpp_SetText(rTxt);
}


void FAndroidKeyboardModule::SetCursor(int iPos)
{
	AndroidThunkCpp_SetCursor(iPos);
}

int FAndroidKeyboardModule::GetCursor()
{
	return AndroidThunkCpp_GetCursor();
}

bool FAndroidKeyboardModule::IsKeyboardOpen()
{
	return AndroidThunkCpp_IsKeyboardOpen();
}

void FAndroidKeyboardModule::HideKeyboard(int ref)
{
	__android_log_print(ANDROID_LOG_INFO, LOG_TAG, "LOOKHERE: ***FAndroidKeyboardModule::HideKeyboard***");
	AndroidThunkCpp_HideKeyboard(ref);
}

int FAndroidKeyboardModule::GetKeyboardHeight()
{
	int iCurrentHeight = AndroidThunkCpp_GetKeyboardHeight();
	if (iCurrentHeight != m_KeyboardHeight)
	{
		__android_log_print(ANDROID_LOG_INFO, LOG_TAG, "LOOKHERE: ***FAndroidKeyboardModule::GetKeyboardHeight New Height = %d***", iCurrentHeight);
		m_KeyboardHeight = iCurrentHeight;
	}
	return m_KeyboardHeight;
}

void FAndroidKeyboardModule::SetText(const FString &rStr)
{
	AndroidThunkCpp_SetText(rStr);
}

void FAndroidKeyboardModule::CPPOnTextChange(const FString &rStr, int ref, bool bDone)
{
	if (m_KeyboardResult)
	{
		__android_log_print(ANDROID_LOG_INFO, LOG_TAG, "LOOKHERE: ***FAndroidKeyboardModule::CPPOnTextChange***");
		m_KeyboardResult(bDone ? AndroidKeyboardResult::Committed : AndroidKeyboardResult::Changed, rStr, ref);
	}
	else
	{
		__android_log_print(ANDROID_LOG_ERROR, LOG_TAG, "LOOKHERE: ***FAndroidKeyboardModule::CPPOnTextChange NO CALLBACK***");
	}
}

#endif


#undef LOCTEXT_NAMESPACE
	
IMPLEMENT_MODULE(FAndroidKeyboardModule, AndroidKeyboard)

#if PLATFORM_ANDROID
void AndroidThunkCpp_OpenKeyboard(int ref, int type)
{
	if (!AndroidThunkJava_ShowKeyboard || !FAndroidKeyboardModule::ENV) return;
	FJavaWrapper::CallVoidMethod(FAndroidKeyboardModule::ENV, FJavaWrapper::GameActivityThis, AndroidThunkJava_ShowKeyboard, ref, type);
}

bool AndroidThunkCpp_IsKeyboardOpen()
{
	if (!AndroidThunkJava_IsKeyboardOpen || !FAndroidKeyboardModule::ENV) return false;
	return FJavaWrapper::CallBooleanMethod(FAndroidKeyboardModule::ENV, FJavaWrapper::GameActivityThis, AndroidThunkJava_IsKeyboardOpen);
}

void AndroidThunkCpp_HideKeyboard(int ref)
{
	if (!AndroidThunkJava_HideKeyboard || !FAndroidKeyboardModule::ENV) return;
	FJavaWrapper::CallVoidMethod(FAndroidKeyboardModule::ENV, FJavaWrapper::GameActivityThis, AndroidThunkJava_HideKeyboard, ref);
}

int AndroidThunkCpp_GetKeyboardHeight()
{
	if (!AndroidThunkJava_GetKeyboardHeight || !FAndroidKeyboardModule::ENV) 
	{
		return 0;
	}
	return FJavaWrapper::CallIntMethod(FAndroidKeyboardModule::ENV, FJavaWrapper::GameActivityThis, AndroidThunkJava_GetKeyboardHeight);
}

int AndroidThunkCpp_GetCursor()
{
	if (!AndroidThunkJava_GetCursor || !FAndroidKeyboardModule::ENV)
	{
		return 0;
	}
	return FJavaWrapper::CallIntMethod(FAndroidKeyboardModule::ENV, FJavaWrapper::GameActivityThis, AndroidThunkJava_GetCursor);
}

void AndroidThunkCpp_SetCursor(int iPos)
{
	if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
	{
		FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, AndroidThunkJava_SetCursor, iPos);
	}
}

void AndroidThunkCpp_SetText(const FString& txt)
{
	if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
	{
		jstring txtJava = Env->NewStringUTF(TCHAR_TO_UTF8(*txt));
		FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, AndroidThunkJava_SetText, txtJava);
		Env->DeleteLocalRef(txtJava);
	}
}

extern "C" void Java_com_epicgames_ue4_GameActivity_CPPOnTextChange(JNIEnv* LocalJNIEnv, jobject thiz, jstring _txt, jint ref, jboolean bdone)
{
	if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
	{
		const char* txtChars = Env->GetStringUTFChars(_txt, 0);
		FString txt(UTF8_TO_TCHAR(txtChars));
		Env->ReleaseStringUTFChars(_txt, txtChars);
		FAndroidKeyboardModule::CPPOnTextChange(txt, ref, bdone);
	}
}

 #endif
